<?php

namespace Larareal\Realestate\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Larareal\Realestate\Http\Requests\PropertyCategoryRequest;
use Larareal\Realestate\Interfaces\PropertyCategoryRepositoryInterface;
use Larareal\Realestate\Models\PropertyCategory;

/**
 * Resource controller class for property_category.
 */
class PropertyCategoryResourceController extends BaseController
{

    /**
     * Initialize property_category resource controller.
     *
     * @param type PropertyCategoryRepositoryInterface $property_category
     *
     * @return null
     */
    public function __construct(PropertyCategoryRepositoryInterface $property_category)
    {
        parent::__construct();
        $this->repository = $property_category;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Larareal\Realestate\Repositories\Criteria\PropertyCategoryResourceCriteria::class);
    }

    /**
     * Display a list of property_category.
     *
     * @return Response
     */
    public function index(PropertyCategoryRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Larareal\Realestate\Repositories\Presenter\PropertyCategoryPresenter::class)
                ->$function();
        }

        $property_categories = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('realestate::property_category.names'))
            ->view('realestate::property_category.index', true)
            ->data(compact('property_categories', 'view'))
            ->output();
    }

    /**
     * Display property_category.
     *
     * @param Request $request
     * @param Model   $property_category
     *
     * @return Response
     */
    public function show(PropertyCategoryRequest $request, PropertyCategory $property_category)
    {

        if ($property_category->exists) {
            $view = 'realestate::property_category.show';
        } else {
            $view = 'realestate::property_category.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('realestate::property_category.name'))
            ->data(compact('property_category'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new property_category.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(PropertyCategoryRequest $request)
    {

        $property_category = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('realestate::property_category.name')) 
            ->view('realestate::property_category.create', true) 
            ->data(compact('property_category'))
            ->output();
    }

    /**
     * Create new property_category.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(PropertyCategoryRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $property_category                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('realestate::property_category.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('realestate/property_category/' . $property_category->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/realestate/property_category'))
                ->redirect();
        }

    }

    /**
     * Show property_category for editing.
     *
     * @param Request $request
     * @param Model   $property_category
     *
     * @return Response
     */
    public function edit(PropertyCategoryRequest $request, PropertyCategory $property_category)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('realestate::property_category.name'))
            ->view('realestate::property_category.edit', true)
            ->data(compact('property_category'))
            ->output();
    }

    /**
     * Update the property_category.
     *
     * @param Request $request
     * @param Model   $property_category
     *
     * @return Response
     */
    public function update(PropertyCategoryRequest $request, PropertyCategory $property_category)
    {
        try {
            $attributes = $request->all();

            $property_category->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('realestate::property_category.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('realestate/property_category/' . $property_category->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('realestate/property_category/' . $property_category->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the property_category.
     *
     * @param Model   $property_category
     *
     * @return Response
     */
    public function destroy(PropertyCategoryRequest $request, PropertyCategory $property_category)
    {
        try {

            $property_category->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('realestate::property_category.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('realestate/property_category/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('realestate/property_category/' . $property_category->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple property_category.
     *
     * @param Model   $property_category
     *
     * @return Response
     */
    public function delete(PropertyCategoryRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('realestate::property_category.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('realestate/property_category'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/realestate/property_category'))
                ->redirect();
        }

    }

    /**
     * Restore deleted property_categories.
     *
     * @param Model   $property_category
     *
     * @return Response
     */
    public function restore(PropertyCategoryRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('realestate::property_category.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/realestate/property_category'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/realestate/property_category/'))
                ->redirect();
        }

    }

}
